@@ -1,4 +1,4 @@
-#!/usr/bin/env python3
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
 import asyncio
@@ -16,17 +16,10 @@ from functools import partial
 from typing import Any, Callable, Dict, Generator, List, NamedTuple, Optional
 
 from kitty.constants import is_macos
-from kitty.fast_data_types import (
-    FILE_TRANSFER_CODE, close_tty, normal_tty, open_tty,
-    parse_input_from_terminal, raw_tty
-)
-from kitty.key_encoding import (
-    ALT, CTRL, SHIFT, backspace_key, decode_key_event, enter_key
-)
+from kitty.fast_data_types import FILE_TRANSFER_CODE, close_tty, normal_tty, open_tty, parse_input_from_terminal, raw_tty
+from kitty.key_encoding import ALT, CTRL, SHIFT, backspace_key, decode_key_event, enter_key
 from kitty.typing import ImageManagerType, KeyEventType, Protocol
-from kitty.utils import (
-    ScreenSize, ScreenSizeGetter, screen_size_function, write_all
-)
+from kitty.utils import ScreenSize, ScreenSizeGetter, screen_size_function, write_all
 
 from .handler import Handler
 from .operations import MouseTracking, init_state, reset_state
@@ -109,17 +102,17 @@ class TermManager:
 
 
 class MouseButton(IntFlag):
-    NONE, LEFT, MIDDLE, RIGHT, FOURTH, FIFTH = 0, 1, 2, 4, 8, 16
-    WHEEL_UP, WHEEL_DOWN = -1, -2
+    NONE, LEFT, MIDDLE, RIGHT, FOURTH, FIFTH, SIXTH, SEVENTH = 0, 1, 2, 4, 8, 16, 32, 64
+    WHEEL_UP, WHEEL_DOWN, WHEEL_LEFT, WHEEL_RIGHT = -1, -2, -4, -8
 
 
-bmap = {0: MouseButton.LEFT, 1: MouseButton.MIDDLE, 2: MouseButton.RIGHT}
+bmap = MouseButton.LEFT, MouseButton.MIDDLE, MouseButton.RIGHT
+ebmap = MouseButton.FOURTH, MouseButton.FIFTH, MouseButton.SIXTH, MouseButton.SEVENTH
+wbmap = MouseButton.WHEEL_UP, MouseButton.WHEEL_DOWN, MouseButton.WHEEL_LEFT, MouseButton.WHEEL_RIGHT
 SHIFT_INDICATOR = 1 << 2
 ALT_INDICATOR = 1 << 3
 CTRL_INDICATOR = 1 << 4
 MOTION_INDICATOR = 1 << 5
-SCROLL_BUTTON_INDICATOR = 1 << 6
-EXTRA_BUTTON_INDICATOR = 1 << 7
 
 
 class EventType(Enum):
@@ -150,13 +143,12 @@ def decode_sgr_mouse(text: str, screen_size: ScreenSize) -> MouseEvent:
     typ = EventType.RELEASE if m == 'm' else (EventType.MOVE if cb & MOTION_INDICATOR else EventType.PRESS)
     buttons: MouseButton = MouseButton.NONE
     cb3 = cb & 3
-    if cb3 != 3:
-        if cb & SCROLL_BUTTON_INDICATOR:
-            buttons = MouseButton.WHEEL_DOWN if cb3 & 1 else MouseButton.WHEEL_UP
-        elif cb & EXTRA_BUTTON_INDICATOR:
-            buttons |= MouseButton.FIFTH if cb3 & 1 else MouseButton.FOURTH
-        else:
-            buttons |= bmap[cb3]
+    if cb >= 128:
+        buttons |= ebmap[cb3]
+    elif cb >= 64:
+        buttons |= wbmap[cb3]
+    elif cb3 < 3:
+        buttons |= bmap[cb3]
     mods = 0
     if cb & SHIFT_INDICATOR:
         mods |= SHIFT
@@ -331,13 +323,8 @@ class Loop:
             except Exception:
                 pass
             else:
-                if k.matches('ctrl+c'):
-                    self.handler.on_interrupt()
-                    return
-                if k.matches('ctrl+d'):
-                    self.handler.on_eot()
-                    return
-                self.handler.on_key_event(k)
+                if not self.handler.perform_default_key_action(k):
+                    self.handler.on_key_event(k)
 
     def _on_pm(self, pm: str) -> None:
         pass
